#version 330

layout (std140) uniform Matrices
{
	mat4 m_pvm;
};

uniform sampler2D particleUnit;

in vec2 position;	// texture space

out vec4 particle;

// Pseudo random function
float rand(vec2 seed)
{
	return fract(sin(dot(seed.xy, vec2(12.9898f, 78.233f))) * 43758.5453f);
}

//----------------------------------------------------------------
// TODO:
// 1) Load particle data from texture.
// 2) Update the positions using a pseudorandom generator: (rand(position*123.456f)*2.f-1.f)*position_speed;
// 3) Decrease the lifetime of the particle.
// 4) Restart dead particles
// 5) Send the data to fragment shader.
// 6) Set point size so that it updates a single texel.
// 7) Compute gl_Position so that texel position gets updated, mind the position/texture ranges.
//----------------------------------------------------------------

void main()
{
	float position_speed = 0.005f;
	float lifetime_speed = 0.002f;

	// Get particle data from last texture
	vec4 data = texture(particleUnit, position);

	// Each particle has the same translation vector. Pseudo random function is used to randomly distribute the particles over the screen
	data.x += (rand(position*123.456f)*2.f-1.f)*position_speed;
	data.y += (rand(position*234.567f)*2.f-1.f)*position_speed;
	data.z += (rand(position*345.678f)*2.f-1.f)*position_speed;

	// Decrease the lifetime
	data.w -= lifetime_speed;

	// Particle died, create new one
	if(data.w < 0.f)
	{
		data.x = 0.f;
		data.y = 0.f;
		data.z = 0.f;
		data.w = clamp(1.f - rand(position*456.789f), 0.3f, 1.f);
	}
	particle = data; // Sent to fragment shader

	// Only write to one texel
	gl_PointSize = 1.f;

	// Translate to NDC, which is between -1.f to +1.f
	ivec2 texSize = textureSize(particleUnit, 0);
	gl_Position = vec4(vec2(-1.f+1.f/texSize.x, -1.f+1.f/texSize.y) + position*2.f, 0.f, 1.f);
}